
#ifndef BULANEC_H

#define BULANEC_H

#include "zbrane.h"

#define BULANEC_SIRKA 30
#define BULANEC_VYSKA 30

#define GRAFIKA_BULANEC_SIRKA 40
#define GRAFIKA_BULANEC_VYSKA 54

#define MAX_ANIMACIA 8

#define KRIZ_SIZE_X 30
#define KRIZ_SIZE_Y 40

#define MAX_POCET_STRIEL 5
#define MAX_POCET_MIN 3

#define KROK 4

#define CAS_TRVANIA_BONUSU 500

#define POCET_KLIKNUTI_KLAVESY 2 

#define SURADNICA_ZIADNA -1 

#define CAS_NEMOZE_STRIELAT_BRKOVNICA 40
#define CAS_NEMOZE_STRIELAT_OSTATNE_ZBRANE 10
#define CAS_NABYJA_SA_ZASOBNIK 100
#define CAS_NEMOZE_PREHODIT_ZBRAN 10
#define CAS_NEMOZE_SPUSTIT_PAUZU 10

#define SOM_AI 1
#define SOM_OVLADANY_CLOVEKOM 0
 
#define POCET_FARIEB_BULANCOV 4

typedef struct s_bulanec
{
	int id; //idetifikacne cislo
	int x; //X-ova suradnica v bojisku
	int y; //Y-ova suradnica v bojisku
	int otoceny; // 8-na sever 6-na vychod 4-na zapad 2-ja juh
	int naboje[POCET_ZBRANI]; //pocet nabojov jednotliivych zbrani
	int zbran; //typ zbrane
	int mozeStrielat; // 0-ano (10..1 nabyja sa dalsi naboj)
	int nabijaSaZasobnik; //0-nie | viac ano
	int mrtvy; // 0-nie | 1 a viac - ano
	int mozePrehoditZbran; //?
	int score; //score bulanca
	int vidiet; //je viditelny? (0=ano inac nie)
	char meno[STR_SIZE]; //jmeno bulanka
	int animaciaKrokov;//citac animacie krokov
	int bonus; //bonus, ktory tuxanec ma
	int casBonus; //cas dokedy trva bonus
	int jeAI; //je tuxance ovladany AI?
} bulanec;

/*
 * Nacita obrazky tuxancov, vola sa prave raz pri spustani programu
 */
extern void initBulanec();

extern bulanec *getbulanec(int n);

extern int getPocetBulancov();

/*
 * nakresli daneho bulanca na obrazovku
 * parametre:
 * *s - ponter na strukturu daneho bulanca
 */
extern void kresliBulanec(bulanec *s);

/*
 * Zistuje ci je kolzia medzi tuxancom *p a ostatnymi
 * tuxancami.
 *
 * parametre:
 * *p - pointer na strukturu tuxanca
 * px py - hodnoty, ktore sa maju pricitat k suradnici tuxanca
 * (pouziva sa iba vo funkcii, suradnice ulozene v strukture sa nezmenia)
 *
 * navratova hodnota:
 * 0 - klzia nenastala
 * 1 - kolizia nastala
 */
extern int koliziaBulancov(bulanec *p,int px,int py);

/*
 * Zistuje ci je kolzia medzi tuxancom *p a
 * strelami ulozenymi v pointeri na strukturu zoznamStriel.
 * Ak je tuxanec zasiahnuty, zstrel sa vyradi zo zonamu stiel,
 * nastavy sa priznak smrti na 1 (p->smrt=1), prehra sa zvuk.
 * Pocet kol sa zvysi o 1 (kolo++) ak pocetKol dosiahol maximalny
 * pocet zavola sa funkcia quitLevel();
 *
 * parametre:
 * *p - pointer na strukturu tuxanca
 */
extern int koliziaStrelaBulanec(bulanec *p);

/*
 * Zmeni suradnice bulanca podla daneho smer (nekontroluje kolizie)
 * a inkriminuje citac animacie.

 */
extern void pohybBulanec(bulanec *s,int smer);

/*
 * Promenna armady bulancov :-)
 */
extern bulanec **armadaBulancov;

extern bulanec* newBulanec(int x,int y,int id);

/*
 * Stara sa o casovace.
 *
 * Ak je tuxanec neviditelny videt>0 odpocitava videt-- az do kedy sa videt=0
 *
 * Ak tuxanec nemoze strielat (lebo pred chvilou vystrelil) mozeStrielat>0 odpocitava
 * az dokedy sa mozeStrielat=0
 *
 * Ak tuxanec nemoze zmenit zbran (lebo pred chvilou uz zbran zmenil) mozePrehoditZbran>0 odpocitava
 * az dokedy sa mozePrehoditZbran=0
 *
 * Ak ma tuxanec bonus odpocitava sa cas dokedy ho moze este mat. Ak ma bonus BONUS_DUCH nezoberie
 * sa mu hned (mohol bysa zeseknut o prekazku) ale az ked z brekazky vylezie. (ak bonus uplynul
 * a je este stale v prekazke tak nemoze strielat)
 *
 * Ak sa zisti, ze tuxanec nema _ziadnu_ zbran zacne sa nabyjat zasobnik. ostom sa mu doda
 * MAX_POCET_STRIEL do zbrane PISTOL
 *
 * Ked bulanec zomrie mrtvy>0 pripocitava sa cas mrtvy++ az dokedy nebude mat urcitu hodnotu
 * a vytvori sa na nahodnom mieste.
 */
extern void akciaBulanec(bulanec *s);

/*
 * bulanec si vymeni zbran za inu. (ak nejaku inu ma)
 */
extern void prehodZbran(bulanec *b);

/*
 * Zisti skutocne rozmery tuxanca, s ktorymi sa zisruju kolizie
 */
extern void getRozmery(bulanec *p,int *x,int *y,int *w,int *h);

/*
 * Zistia sa rozmery sa suradnice grafiky tuxanca.
 */
extern void getRozmeryGrafika(bulanec *p,int *x,int *y,int *w,int *h);

/*
 * Odstrani tuxanca z pamate
 */
extern void destroyBulanec(bulanec *s);

/*
 * Z pamate sa uvolni grafika.
 */
extern void quitBulanec();

#endif
