
#ifndef INTERFACE_H

#define INTERFACE_H

#include <stdio.h>
#include <string.h>
#include <time.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>

#include <SDL.h>
#include <SDL_image.h>
#include <SDL_ttf.h>
#include <SDL_thread.h>
#include <SDL_net.h>

#ifdef WIN32
#include "w32func.h"
#endif

#ifndef NO_SOUND
#include <SDL_mixer.h>
#endif

#define SDL_SUBSYSTEMS SDL_INIT_VIDEO|SDL_INIT_TIMER|SDL_DOUBLEBUF

//rozlisenie obrazovky
#define WINDOW_SIZE_X 640
#define WINDOW_SIZE_Y 480

#define HRACIA_PLOCHA_SIZE_X 640
#define HRACIA_PLOCHA_SIZE_Y 480

//napis na okne
#define WINDOW_TITLE "Tuxanci"
#define WIN_BPP 0
#define USR_EVT_TIMER 0

//interval pri ktorom sa spusta akcia();
#define INTERVAL 25

#define STR_SIZE 256

#define ZNAK_KONIEC_RIADKA 0xA

#define VRSTVA_POD_TUXANCOM 0
#define VRSTVA_DIERY 0
#define VRSTVA_S_TUXANCOM 1
#define VRSTVA_NAD_TUXANCOM 2

#define PREKRESLI_RAMEC 1
#define NEPREKRESLI_RAMEC 0

#define HORE 8
#define VPRAVO 6
#define VLAVO 4
#define DOLE 2
#define STREL 0

#define MAIN_MENU 1
#define HRA_SA 2
#define VYBER_LEVEL 3
#define NASTAVENIA 4
#define NASTAVENIA_SIETE 5
#define TABULKA 6
#define AUTORY 7
#define VYHODNOTENIE 8
#define KONIEC 9

#define NONE 0
#define SERVER 1
#define CLIENT 2

#define MAX_PLAYERS 5

#include "directories.h"
#include "config.h"

typedef struct s_prekresli
{
	int x;
	int y;
	int w;
	int h;
	struct s_prekresli *next;
} prekresli;

typedef struct s_myimage
{
	int x; //suradnice obrazka na obrazovke
	int y;
	int w; //rozmery obrazka na obrazovke
	int h;
	int px; //suradnice casti surfrace
	int py;
	int pw; //rozmeri casti surfrace
	int ph;
	int vrstva; //(0 - pod tuxancami | 1 - na tej istej ako tuxanci | 2 - nad tuxancami)
	SDL_Surface *image; //dane surfrace
	int flagredraw; //ma sa aj prekreslit?
	struct s_myimage *next;
} myimage;

extern SDL_Surface *screen;
extern prekresli *zoznamPrekresli;
extern myimage *zoznamImage;
extern TTF_Font *g_font;

/*
 * Inicializuje SDL k pouzitiu
 */
int init();

char *getDomovskyAdresar();

void vytvorDomovskyAdresar();

char ** loadSubor(char *s);

void destroySubor(char **s);

/*
 * Prida obrazok na vykreslenie do zoznamu **p
 * *img - ponter na urfrace obrazka
 * x,y - suradnice kam sa ma obrazok nakreslit
 * w,h - rozmeri obrazka
 * px,py - suradnice casti *img z ktorej sa bude obrazok vykreslovat
 * ph,ph - rozmeri casti obrazka *img z ktorej sa bude obrazok vykreslovat
 * vrstva - vrstva, na ktorej bude obrazok 
 *          (0 - pod tuxancami | 1 - na tej istej ako tuxanci | 2 - nad tuxancami)
 * flagredraw - ma sa frame aj prekreslit
 */

void pridajImage(myimage **p,SDL_Surface *img,
	int x,int y,int w,int h,
	int px,int py,int pw,int ph,int vrstva,int flagredraw);

/*
 * Vykresli zoznam *p na obrazovku
 */
void zoznamImageShow(myimage **p);

/*
 * prida ramec do zozamu ramcov v okne, ktore sa maju prekresli
 * x,y - suradnice ramca
 * w,h - rozmeri ramca
 */
void pridajPrekresli(prekresli **p,int x,int y,int w,int h);

/*
 * Prekresli tie casti obrazovky, ktore sa nachadzaju
 * v zozname ramocov **p. Po krekresleni sa zozname
 * uvolni.
 */
void akciaPrekresli(prekresli **p);

/*
 * V strede obrazovky zobrazi danu spravu
 */
void zobrazSpravu(char *s);

/*
 * Sprava ktora sa zobrazila sa odstrani
 */
void odstranSpravu();

/*
 * funckia na akreslienie spravy
 */
void kresliSpravu();

/*
 * Stara sa o prekreslenie screny
 * vacsinou spusta dalsie funkcie
 */
void kresli();

/*
 * Prepina z/do okna/fullscreenu
 */
int toggleFullscreen();

/*
 * Zachytava sa spracuvava udalosti zachytene SDL
 */
int eventAction();

/*
 * Susta uzivatelsku funkciu, v intervale 0.25sek
 */
Uint32 TimerCallback(Uint32 interval, void* param);

/*
 * Nacita obrazok v filename,
 * alpha - ma sa pouzit alpha kanal ?
 */
SDL_Surface *loadImage(const char *filename,int alpha);

/*
 * Nastavi cast programu v ktorom sa hrac nachadza :
 * MAIN_MEUN - havne menu hry
 * VYBER_LEVEL - menu na vyber levelu
 * HRA_SA - hracia arena
 */
void setMiesto_v_hre(int x);

/*
 * zavola vsetky funkcie quit*()
 * a ukonci program.
 */
void vsetkoZabal();

#ifdef NET_MULTIPLAYER

extern SDL_mutex *mutex_akcia;

#endif

#endif
