
#include "interface.h"
#include "keytable.h"

static char ** keyTableConfig;
static int keytable[POCET_AKCII][POCET_KLAVES];

static int getKeySDLCode(char *s)
{
	int i;
	char cmd[STR_SIZE];
	char strcode[STR_SIZE];
	char rovnasa[STR_SIZE];
	int numCode;

	for(i=0;keyTableConfig[i]!=NULL;i++)
	{
		if(strncmp(keyTableConfig[i],STR_KEYCODE,strlen(STR_KEYCODE))==0)
		{
			sscanf(keyTableConfig[i],"%s %s %s %d",cmd,strcode,rovnasa,&numCode);
			if(strcmp(strcode,s)==0)return numCode;
		}
	}

	fprintf(stderr,"Kod klavesy %s nenajdeny !\n",s);
	return (-1);
}

static void setKeytable(char *s,int n)
{
	int i;
	char strcode[STR_SIZE];
	char *str;
	int pocet=0;

	for(i=0;keyTableConfig[i]!=NULL;i++)
	{
		if(strncmp(keyTableConfig[i],s,strlen(s))==0)
		{
			str=keyTableConfig[i];

			while((str=strstr(str,"SDLK"))!=NULL)
			{
				sscanf(str,"%s",strcode);
				str+=(int)4;
				keytable[n][pocet]=getKeySDLCode(strcode);
				//printf("keytable[%d][%d]=%d\n",n,pocet,keytable[n][pocet]);
				pocet++;

				if(pocet==POCET_KLAVES)
				{
					printf("prilis velka klaves definovanych v %s\n",keyTableConfig[i]);
					return ;
				}
			}
		}
	}

	if(pocet==0)fprintf(stderr,"Kod akcie %s nenajdeny !\n",s);
}

void prekopirujSubor(char *filein,char *fileout)
{
	FILE *fin;
	FILE *fout;
	int velkost=0;
	char *p;

#ifdef WIN32
	velkost = p_GetFileSize(filein);
#else
	struct stat buf;

	if(lstat(filein,&buf)<0)
	{
		fprintf(stderr,"CHYBA: nemozem ziskat status suboru %s!\n",filein);
		return;
	}

	velkost=buf.st_size;
#endif

	if((fin = fopen(filein,"rb"))==NULL)
	{
		fprintf(stderr,"CHYBA: nemozem otvorit subor %s pre citanie\n",filein);
		return;
	}

	p = malloc(velkost*sizeof(char));

	if(fread(p,velkost*sizeof(char),1,fin)!=1)
	{
		fprintf(stderr,"CHYBA: nemozem nacitat udahe zo suboru %s\n",filein);
		fclose(fin);
		free(p);
		return;
	}

	fclose(fin);

	if((fout = fopen(fileout,"wb"))==NULL)
	{
		fprintf(stderr,"CHYBA: nemozem otvorit subor %s pre zapis\n",fileout);
		free(p);
		return;
	}

	if(fwrite(p,velkost*sizeof(char),1,fout)!=1)
	{
		fprintf(stderr,"CHYBA: nemozem zapisat udaje do suboru %s\n",fileout);
		fclose(fout);
		free(p);
		return;
	}

	fclose(fout);

	free(p);
}

void initKeytable()
{
	char cesta[STR_SIZE];
	char cmd[STR_SIZE];
	int i,j;

	sprintf(cesta,"%s/keytable.conf",getDomovskyAdresar());

	if(access(cesta,F_OK)!=0)
	{
		sprintf(cmd,"%s/keytable.conf",getDomovskyAdresar());
		prekopirujSubor(PATH_DATA "keytable.conf",cmd);
		printf("vytvaram subor %s\n",cmd);
	}

	keyTableConfig=loadSubor(cesta);

	for(i=0;i<POCET_AKCII;i++)
		for(j=0;j<POCET_KLAVES;j++)
			keytable[i][j]=-1;

	setKeytable(STR_TUX_RIGHT_MOVE_UP,KLAVES_BULANEC_VPRAVO_POHYB_HORE);
	setKeytable(STR_TUX_RIGHT_MOVE_RIGHT,KLAVES_BULANEC_VPRAVO_POHYB_VPRAVO);
	setKeytable(STR_TUX_RIGHT_MOVE_LEFT,KLAVES_BULANEC_VPRAVO_POHYB_VLAVO);
	setKeytable(STR_TUX_RIGHT_MOVE_DOWN,KLAVES_BULANEC_VPRAVO_POHYB_DOLE);
	setKeytable(STR_TUX_RIGHT_SHOOT,KLAVES_BULANEC_VPRAVO_STRIELAJ);
	setKeytable(STR_TUX_RIGHT_SWITCH_WEAPON,KLAVES_BULANEC_VPRAVO_ZMEN_ZBRAN);

	setKeytable(STR_TUX_LEFT_MOVE_UP,KLAVES_BULANEC_VLAVO_POHYB_HORE);
	setKeytable(STR_TUX_LEFT_MOVE_RIGHT,KLAVES_BULANEC_VLAVO_POHYB_VPRAVO);
	setKeytable(STR_TUX_LEFT_MOVE_LEFT,KLAVES_BULANEC_VLAVO_POHYB_VLAVO);
	setKeytable(STR_TUX_LEFT_MOVE_DOWN,KLAVES_BULANEC_VLAVO_POHYB_DOLE);
	setKeytable(STR_TUX_LEFT_SHOOT,KLAVES_BULANEC_VLAVO_STRIELAJ);
	setKeytable(STR_TUX_LEFT_SWITCH_WEAPON,KLAVES_BULANEC_VLAVO_ZMEN_ZBRAN);
}

int isStlacilKlaves(int p)
{
	int i;
	Uint8 *mapa;
	mapa=SDL_GetKeyState(NULL);

	for(i=0;i<POCET_KLAVES;i++)
		if( mapa[ keytable[p][i] ]==SDL_PRESSED )return 1;

	return 0;
}

void quitKeytable()
{
	destroySubor(keyTableConfig);
}
